﻿using UnityEngine;
using HarmonyLib;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

public class GarbageSucker : Mod
{
    private readonly string HARMONY_ID = "TeigRolle.GarbageSucker";
    private Harmony harmonyInstance;

    public static AssetBundle assetbundle;

    public IEnumerator Start()
    {
        harmonyInstance = new Harmony(HARMONY_ID);
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());

        HNotification notification = FindObjectOfType<HNotify>().AddNotification(HNotify.NotificationType.spinning, "Loading Garbage Sucker...");

        AssetBundleCreateRequest request = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("garbagesucker.assets"));
        yield return request;
        assetbundle = request.assetBundle;

        List<Item_Base> list = Traverse.Create(typeof(ItemManager)).Field("allAvailableItems").GetValue<List<Item_Base>>();
        Item_Base placeable_garbagesucker = (Item_Base)assetbundle.LoadAsset<ScriptableObject>("placeable_garbagesucker");
        GarbageSuckerObject obj = placeable_garbagesucker.settings_buildable.GetBlockPrefab(0).gameObject.AddComponent<GarbageSuckerObject>();
        obj.gameObject.transform.Find("WaterTankFake").gameObject.AddComponent<Tank>();
        obj.GetComponent<Sprinkler>().waterTank = obj.GetComponentInChildren<Tank>();
        placeable_garbagesucker.settings_buildable.GetBlockPrefab(0).transform.Find("RaycastCollider").gameObject.AddComponent<GBRaycastReciever>();
        placeable_garbagesucker.settings_buildable.GetBlockPrefab(0).GetComponent<Block>().networkedIDBehaviour = obj;

        Traverse.Create(placeable_garbagesucker.settings_buildable.GetBlockPrefab(0).GetComponentInChildren<Battery>()).Field("networkBehaviourID").SetValue(obj);
        placeable_garbagesucker.Initialize(1181, "placeable_garbagesucker", 1);
        list.Add(placeable_garbagesucker);

        Traverse.Create(typeof(ItemManager)).Field("allAvailableItems").SetValue(list);

        Debug.Log("s : "+placeable_garbagesucker);
        RAPI.AddItemToBlockQuadType(placeable_garbagesucker, RBlockQuadType.quad_itemnet_empty);
        //RAPI.AddItemToBlockQuadType(placeable_garbagesucker, RBlockQuadType.quad_itemnet_empty);

        Debug.Log("Garbage Sucker has been loaded!");

        notification.Close();
    }

    public void OnModUnload()
    {
        harmonyInstance.UnpatchAll(HARMONY_ID);
    }
}

public class GarbageSuckerObject : MonoBehaviour_ID_Network
{

    public Battery battery;

    private List<ItemNet> nets;
    private List<Storage_Small> storages;

    public int secsBetweenRecharges = 5;

    public float range = 3f;

    public float refreshIntervall = 5f;
    private float timer;

    LayerMask everything = ~0;

    Transform wheel;

    float rotationSpeed = 1000;

    void Start()
    {
        wheel = transform.Find("NewModel").Find("PaintMill_Wheel");
        battery = GetComponentInChildren<Battery>();
    }

    void Update()
    {

        timer += Time.deltaTime;

        if (!battery.BatterySlotIsEmpty && battery.BatteryUses > 0)
            wheel.Rotate(Vector3.right * rotationSpeed * Time.deltaTime, Space.Self);

        if (timer > refreshIntervall)
        {
            timer -= refreshIntervall;

            if (battery.BatterySlotIsEmpty || battery.BatteryUses == 0)
                return;

            nets = new List<ItemNet>();
            storages = new List<Storage_Small>();

            Collider[] cols = Physics.OverlapSphere(transform.position, range, everything);


            foreach (Collider col in cols)
            {
                if (col.gameObject.GetComponent<ItemNet>() != null)
                {
                    if (Vector3.Distance(transform.position, col.gameObject.transform.position) < .3f)
                    {
                        nets.Add(col.gameObject.GetComponentInChildren<ItemNet>());
                    }
                }
                if (col.gameObject.GetComponent<Storage_Small>() != null)
                {
                    storages.Add(col.gameObject.GetComponentInChildren<Storage_Small>());
                }
            }

            if (storages.Count > 0 && nets.Count > 0)
            {
                foreach (ItemNet net in nets)
                {
                    List<PickupItem_Networked> remove = new List<PickupItem_Networked>();

                    foreach (PickupItem_Networked item in net.itemCollector.collectedItems)
                    {
                        bool found = false;
                        foreach (Storage_Small st in storages)
                        {
                            if (!found)
                            {
                                MethodInfo meth = st.GetInventoryReference().GetType().GetMethod("FindSuitableSlot", BindingFlags.NonPublic | BindingFlags.Instance);
                                Slot slod = (Slot)meth.Invoke(st.GetInventoryReference(), new object[] { item.PickupItem.itemInstance.baseItem });

                                if (slod != null)
                                {

                                    if (item == null)
                                    {
                                        return;
                                    }

                                    Collider[] componentsInChildren = item.GetComponentsInChildren<Collider>();
                                    for (int i = 0; i < componentsInChildren.Length; i++)
                                    {
                                        componentsInChildren[i].enabled = true;
                                    }
                                    item.GetComponent<WaterFloatSemih2>().enabled = true;

                                    if (item.PickupItem.itemInstance.Valid)
                                    {
                                        st.GetInventoryReference().AddItem(item.PickupItem.itemInstance, true);
                                        return;
                                    }
                                    else
                                    {
                                        foreach (Cost cost in item.PickupItem.yieldHandler.yieldAsset.yieldAssets)
                                        {
                                            st.GetInventoryReference().AddItem(cost.item.UniqueName, cost.amount);
                                        }
                                        if (item.PickupItem.dropper != null)
                                        {
                                            Item_Base[] randomItems = item.PickupItem.dropper.GetRandomItems();
                                            for (int i = 0; i < randomItems.Length; i++)
                                            {
                                                if (randomItems[i] != null)
                                                {
                                                    st.GetInventoryReference().AddItem(randomItems[i].UniqueName, 1);
                                                }
                                            }
                                        }
                                    }
                                    battery.RemoveBatteryUsesNetworked(1);
                                    remove.Add(item);
                                    PickupObjectManager.RemovePickupItem(item);

                                    found = true;
                                }
                            }
                        }
                    }
                    foreach (PickupItem_Networked netItem in remove)
                    {
                        net.itemCollector.collectedItems.Remove(netItem);
                    }
                }
            }
        }
    }

    protected override void OnDestroy()
    {
        //if (battery != null && !battery.BatterySlotIsEmpty)
        //RAPI.GetLocalPlayer().Inventory.AddItem(this.battery.GetBatteryInstance(), true);
        base.OnDestroy();
        NetworkIDManager.RemoveNetworkID(this);
    }

    public void OnBlockPlaced()
    {
        NetworkIDManager.AddNetworkID(this);
    }
}

public class GBRaycastReciever : MonoBehaviour_ID_Network, IRaycastable
{
    private Network_Player localPlayer;
    private CanvasHelper canvas;
    private bool showingText;
    public Battery battery;

    public void Start()
    {
        battery = transform.parent.GetComponentInChildren<Battery>();
    }

    public void OnIsRayed()
    {
        if (battery == null)
            return;

        if (this.canvas == null)
        {
            this.canvas = ComponentManager<CanvasHelper>.Value;
        }
        if (this.localPlayer == null)
        {
            this.localPlayer = ComponentManager<Network_Player>.Value;
        }
        if (CanvasHelper.ActiveMenu != MenuType.None || !Helper.LocalPlayerIsWithinDistance(base.transform.position, Player.UseDistance))
        {
            if (this.showingText)
            {
                this.canvas.displayTextManager.HideDisplayTexts();
                this.showingText = false;
            }
            return;
        }
        if (!battery.BatterySlotIsEmpty)
        {
            if (battery.GetBatteryInstance().Uses > 0)
            {
                float percentage = (float)battery.GetBatteryInstance().Uses / (float)ItemManager.GetItemByName("Battery").MaxUses;
                this.canvas.displayTextManager.ShowText(Mathf.Round(percentage * 100).ToString() + " %", 0, true, 0);
                this.showingText = true;
            }
            else
            {
                this.canvas.displayTextManager.ShowText("Battery is empty", 0, true, 0);
                this.showingText = true;
            }
        }
    }

    public void OnRayEnter()
    {
    }

    public void OnRayExit()
    {
    }
}


[HarmonyPatch(typeof(RemovePlaceables))]
[HarmonyPatch("ReturnItemsFromBlock")]
static class RemovePlaceables_ReturnItemsFromBlock_Patch
{
    static bool Prefix(ref bool __result, Block block, Network_Player player)
    {

        Debug.Log("yeeet");

        if (block.buildableItem.UniqueIndex == 1181)
        {
            Sprinkler componentInChildren2 = block.gameObject.GetComponentInChildren<Sprinkler>();
            if (componentInChildren2 != null && !componentInChildren2.battery.BatterySlotIsEmpty)
            {
                componentInChildren2.battery.Take(player, componentInChildren2.battery.BatteryUses);
                __result = true;
                return false;
            }
        }
        __result = false;
        return true;

    }
    public static bool ReturnItemsFromBlock(Block block, Network_Player player)
    {
        if (block.itemToReturnOnDestroy != null)
        {
            int uniqueIndex = block.buildableItem.UniqueIndex;
            if (uniqueIndex <= 172)
            {
                if (uniqueIndex <= 113)
                {
                    if (uniqueIndex != 42)
                    {
                        if (uniqueIndex != 113)
                        {
                            return true;
                        }
                        ResearchTable component = block.gameObject.GetComponent<ResearchTable>();
                        if (component != null)
                        {
                            Slot slot = component.InventoryReference.GetSlot(0);
                            if (slot != null && !slot.IsEmpty)
                            {
                                ItemInstance itemInstance = slot.itemInstance;
                                player.Inventory.AddItem(itemInstance, true);
                            }
                        }
                        return true;
                    }
                }
                else if (uniqueIndex != 140)
                {
                    if (uniqueIndex != 172)
                    {
                        return true;
                    }
                    TrophyTextwriter component2 = block.gameObject.GetComponent<TrophyTextwriter>();
                    return !(component2 == null) && !component2.GetIsBeingEdited();
                }
                Storage_Small component3 = block.gameObject.GetComponent<Storage_Small>();
                if (component3 != null)
                {
                    if (component3.IsOpen)
                    {
                        return false;
                    }
                    Inventory inventoryReference = component3.GetInventoryReference();
                    for (int i = 0; i < inventoryReference.GetSlotCount(); i++)
                    {
                        Slot slot2 = inventoryReference.GetSlot(i);
                        if (!slot2.IsEmpty)
                        {
                            player.Inventory.AddItem(slot2.itemInstance, true);
                        }
                    }
                    return true;
                }
            }
            else if (uniqueIndex <= 197)
            {
                if (uniqueIndex == 173)
                {
                    Reciever component4 = block.gameObject.GetComponent<Reciever>();
                    if (component4 != null && !component4.battery.BatterySlotIsEmpty)
                    {
                        component4.battery.Take(player, component4.battery.BatteryUses);
                    }
                    return true;
                }
                if (uniqueIndex - 195 <= 2)
                {
                    TrophyHolder componentInChildren = block.gameObject.GetComponentInChildren<TrophyHolder>();
                    if (componentInChildren == null)
                    {
                        return false;
                    }
                    TrophyTextwriter componentInParent = componentInChildren.GetComponentInParent<TrophyTextwriter>();
                    if (componentInParent == null)
                    {
                        return false;
                    }
                    if (componentInParent.GetIsBeingEdited())
                    {
                        return false;
                    }
                    Item_Base currentItem = componentInChildren.GetCurrentItem();
                    if (currentItem != null)
                    {
                        player.Inventory.AddItem(currentItem.UniqueName, 1);
                    }
                    return true;
                }
            }
            else if (uniqueIndex != 257)
            {
                if (uniqueIndex == 259)
                {
                    Sprinkler componentInChildren2 = block.gameObject.GetComponentInChildren<Sprinkler>();
                    if (componentInChildren2 != null && !componentInChildren2.battery.BatterySlotIsEmpty)
                    {
                        componentInChildren2.battery.Take(player, componentInChildren2.battery.BatteryUses);
                    }
                }
            }
            else
            {
                TextWriterObject componentInChildren3 = block.gameObject.GetComponentInChildren<TextWriterObject>();
                if (componentInChildren3 == null)
                {
                    return false;
                }
                string text = componentInChildren3.GetText();
                ItemInstance itemInstance2 = new ItemInstance(ItemManager.GetItemByIndex(257), 1, 1, text);
                itemInstance2.exclusiveString = text;
                player.Inventory.AddItem(itemInstance2, true);
                return true;
            }
        }
        else if (!GameModeValueManager.GetCurrentGameModeValue().playerSpecificVariables.unlimitedResources)
        {
            foreach (CostMultiple costMultiple in block.buildableItem.settings_recipe.NewCost)
            {
                int amount = Mathf.CeilToInt((float)costMultiple.amount * 0.5f);
                player.Inventory.AddItem(costMultiple.items[0].UniqueName, amount);
            }
        }
        return true;
    }
}